function definite(kappa, nobs, example, signcontrast, RGrid, hGrid, noiselevel, threshold)
%% SHOW PLOT OF INDICATOR FUNCTION
%  INPUT:
%  kappa - wave number
%  nobs - number of incident/ observation directions
%  example - scatterers that are used
%  signcontrast - 1: Dirichlet scatterers, -1: Neumann scatterers
%  RGrid - side length of region of interest
%  hGrid - step size
%  noiselevel - relative noiselevel
%  threshold - eigenvalues with absolute values below threshold will be
%              discarded

subsampling = 4;  % increase the number of discretization points for the Nystrom method if nobs is too low

thresholdEigenvalues = max([threshold, 100.0*noiselevel]);  % don't use eigenvalues with absolute values smaller than this threshold

% fprintf('\nParameters:\n')
% fprintf('Example: %s\n', example)
% fprintf('Wave number: kappa = %2.2f\n', kappa)
% fprintf('Wave length: lambda = %2.2f\n', 2*pi/kappa)
% fprintf('Number of incident/observation directions: nobs = %2i\n\n', nobs)
% 
% fprintf('Noise level: noiselevel = %1.2e\n\n', noiselevel)
% fprintf('Threshold in monotonicity test: delta = %1.2e\n\n', threshold)

%% simulate farfield data using boundary integral equations
[F, Fcomponents, geometry] = computefarfield(example, kappa, subsampling*nobs, 0);

F = F(1:subsampling:end,1:subsampling:end);


% add noise to the data
if noiselevel > 0
    errmat = rand(size(F))-.5+1i*(rand(size(F))-.5);
    errmat = errmat ./ norm(errmat);
    F = F + errmat * noiselevel * norm(F);
    clear errmat
end


% estimate error in the data
F = F * (2*pi/nobs);  % add quadrature weights of trapezoid rule to obtain matrix w.r.t. char. fnct. basis
EstNormalityError = norm(F*F'-F'*F);  % absoluter Datenfehler
% fprintf('Estimated normality error of F: %1.2e\n', EstNormalityError)

EigF = eig(F);
EigF(abs(EigF)<thresholdEigenvalues) = 0;
% fprintf('\nNumber of negative eigenvalues of Re(F): %i\n', sum(real(EigF)<0));
% fprintf('\nNumber of positive eigenvalues of Re(F): %i\n', sum(real(EigF)>0));

%% simple version of a monotonicity test
% fprintf('\nSimple monotonicity test:\n')

ReF = (F+F')/2;

[XGrid,YGrid] = meshgrid(-RGrid:hGrid:RGrid,-RGrid:hGrid:RGrid);

nt = size(XGrid,1);
I = zeros(nt);
NuOfUsedEigenvalues = zeros(nt);
for iterx=1:nt
    for itery=1:nt
        
        zB = [XGrid(iterx,itery),YGrid(iterx,itery)];  % center of B
        hB = hGrid;  % side length of B = pixel centered at zB
        TB = evaluateTB_circle(kappa,nobs,zB,hB/2);  % test circles
        
        TB = TB * (2*pi/nobs);  % add quadrature weights of trapezoid rule to obtain matrix w.r.t. char. fnct. basis
        
        A = signcontrast*(ReF+signcontrast*TB);
        EigA = eig(A);
        NuOfUsedEigenvalues(iterx,itery)=sum(abs(EigA)>=thresholdEigenvalues);
        EigA(abs(EigA)<thresholdEigenvalues) = 0;
        I(iterx,itery) = sum(real(EigA)>0);  % count nr of negative eigenvalues of ReF-alpha*TB
        
    end
end

% fprintf('Number of eigenvalues used (on average): %2.1f\n\n', mean(mean(NuOfUsedEigenvalues)))

%% show geometry
figure()
nr_of_obstacles = size(geometry,2) / 2;
hold on
for iter = 1:nr_of_obstacles
    plot(geometry(:,2*iter-1), geometry(:,2*iter), '-', 'color', [0, 0, 1], 'linewidth', 2)
end
hold off
axis([-10 10 -10 10])
axis square
set(gca,'Fontsize',20)
set(gca,'XTick',[-10,-5,0,5,10]);
set(gca,'YTick',[-10,-5,0,5,10]);
grid on
box on

% print(['plots/geometry',example,'.eps'],'-depsc');

%%  show reconstruction
figure()
imagesc([-RGrid,RGrid],[-RGrid, RGrid],I)

set(gca,'YDir','normal');
shading flat;
xlim([-RGrid RGrid]); ylim([-RGrid RGrid]);
axis square
colorbar
set(gca,'Fontsize',20)
title(['$k=',num2str(kappa),', N=',num2str(nobs),'$'],'Interpreter','latex')
cb = colorbar;
cbmin = cb.Limits(1);
cbmax = cb.Limits(2);
set(gca, 'clim', cb.Limits);
set(cb, 'ticks', cbmin:cbmax);
set(gca,'XTick',[-10,-5,0,5,10]);
set(gca,'YTick',[-10,-5,0,5,10]);

% if signcontrast==1
%     print(['plots/reconstruction_Dirichlet',example,'_kappa',num2str(kappa),'_nobs',num2str(nobs),'_err',num2str(noiselevel, '%2.0e'),'.eps'],'-depsc');
% elseif signcontrast==-1
%     print(['plots/reconstruction_Neumann',example,'_kappa',num2str(kappa),'_nobs',num2str(nobs),'_err',num2str(noiselevel, '%2.0e'),'.eps'],'-depsc');
% end 
