function rho = estimateradius(F, kappa, signcontrast, zB, threshold, Rmax)
%% ESTIMATERADIUS:
%  INPUT:   
%  F - far field operator
%  kappa - wave number
%  signcontrast - 1: Dirichlet scatterers, -1: Neumann scatterers
%  zB - center of test circle
%           threshold - parameter eta as in formula (6.1) of the article
%  threshold - eigenvalues with absolute values below threshold will be
%              discarded
%  Rmax - size of plots
%  OUTPUT: rho - approximation of the radius of the smallest ball around zB
%                that contains the CSS of farfield
%  EXAMPLE USAGE: rho = estimateradius(farfield,kappa,signcontrast,zB,threshold)


nobs = size(F,1);

ReF = (F+F')/2;

lambda = 2*pi/kappa; % wave length
Rvec = 0:lambda/10:2*Rmax;
nvec = length(Rvec);
qvecA = zeros(nvec,1);
for iter = 1:nvec
   
    TB = evaluateTB_circle(kappa,nobs,zB,Rvec(iter));
    TB = TB * (2*pi/nobs);  % add quadrature weights of trapezoid rule to obtain matrix w.r.t. char. fnct. basis
    
    A = -signcontrast*(ReF+signcontrast*TB);
    eigA = eig(A);
 
    qvecA(iter) = sum(real(eigA)>threshold);
    
end

options = fitoptions('Method','Smooth','SmoothingParam',1*1e-3);
[fit1,gof,out] = fit(Rvec',qvecA,'smooth',options);
[d1,d2] = differentiate(fit1,Rvec');
signed_curvature = d2./(1+d1.^2).^(3/2);
max_curvature = max(signed_curvature,0);
bla = (max_curvature == max(max_curvature));
rho = 1.0 * Rvec(find(bla, 1));


% figure(100)
% plot(Rvec,qvecA,'b','linewidth',2)
% hold on
% plot([rho, rho],[min(qvecA), max(qvecA)],'r--','linewidth',2)
% h = plot(fit1,'k:');
% set(h,'linewidth',2)
% legend('hide')
% hold off
% axis tight
% xlabel('R')
% set(gca,'Fontsize',20)
% box on
